;Music editor for the C64
;Converted from the original 6510+ assembler source code
!source "stdlib.asm"
!to "MusicEditor.prg", cbm
!sl "MusicEditor.map"
!cpu 6502
!ct pet


; Various hard coded variables that need to be tidied
menubuffer1=$0800
menubuffer2=$0900
winbuffer1 =$8c00
winbuffer2 =$8d00
subtext=$8e00

track1 =$4600	; The start of the music data in memory. Must be aligned on $100 bytes.
track2 =$4700
track3 =$4800
envdata=$4900	; Also called FX table data
maxBlocks = $40
loblock =$4a00
hiblock =loblock + maxBlocks
blocks =$4b00
*=track1

backupFXTableSpace = $c100


; This starts at $0801 so that doing a LOAD"*",8 will still work with the default $0801 BASIC start address.
*= BASICSTART
!byte $0c,$08,$0a,$00,$9e		; Line 10 SYS
!convtab pet
;!tx "2560"						; Address for sys start in text
!tx "2304"						; Address for sys start in text
;!byte $33,$32,$37,$36,$38		; In PETSCII
!byte $00,$00,$00,$00
!byte $00,$00,$00,$00			; And a few more zeros for the sake of paranoia and safety.

; This eventually gets overwritten by the menu buffer
*=$0900
	; Copy the screen from one part of memory to be in the RAM underneath the VIC/SID/ROMs
	; Each of these screens were originally saved from data at $4000-47ff and edited by ScreenEdit.asm
	lda #$d0
	ldx #<screen1
	ldy #>screen1
	jsr MoveScreen
	lda #$d8
	ldx #<screen2
	ldy #>screen2
	jsr MoveScreen
	lda #$e0
	ldx #<screen3
	ldy #>screen3
	jsr MoveScreen
	lda #$e8
	ldx #<screen4
	ldy #>screen4
	jsr MoveScreen
	lda #$f0
	ldx #<screen5
	ldy #>screen5
	jsr MoveScreen

	; Jump to main
	jmp MainStart

; A = Start high address of the destination to store
; X = lo source
; Y = hi source
MoveScreen
!zn {
	; Setup the self modifying addresses from the input
	sta toAddr+2
	clc
	adc #8
	sta toChk+1
	txa
	sta fromAddr+1
	tya
	sta fromAddr+2

	; Turn off interrupts, VIC and chargen to enable RAM underneath
	sei
	lda #$34
	sta $01

	; Start the copying until we reach the programmed end
loop1
	ldx #0
loop2
fromAddr
	lda $4000,x				; Self modifying
toAddr
	sta VIC2Sprite0X,x				; Self modifying
	inx
	bne loop2
	inc toAddr+2
	inc fromAddr+2
	lda toAddr+2
toChk
	cmp #$d8				; Self modifying
	bne loop1

	; Enable interupts and normal VIC
	lda #$37
	sta $01
	cli
	rts
}

*=$0a00
!source "MainScreen.asm"

;*=$1800
!source "TrackEdit.asm"

;*=$2000
!source "Storage.asm"

;*=$3000
!source "BlockEdit.asm"

;*=$3a00
!source "Directory.asm"

;*=$3d00
!source "GetKey.asm"

; Must be aligned to $100 bytes
!align 255, 0
RelocMusicPlayerStart
!source "MusicPlayer.asm"


; Must be here at this location
*=$3e00
!source "MusicPlayerIRQ.asm"

; Must be here at this location
; Must be aligned to $100 bytes
*=$3f00
RealMusicPlayerStart
MusicPlayerStart = RealMusicPlayerStart
MusicPlayerPlay = RealMusicPlayerStart + 6
MusicPlayerStop = RealMusicPlayerStart + 9
; MPi: Sort out these hacks to expose via function calls
trcn = RealMusicPlayerStart + $c		; Hack to get the internal .trcn address
!source "MusicPlayer.asm"


; Needed to get sensible envelope values in the default options
; It also allows a bit of music to be loaded as default to test the product if the jsr clearall is commented out.
*= track1
!bin "114MUSDATA.bin"

; Some spare memory after the default music data is loaded. These screens get moved up into higher memory at load time.
screen1
!bin "screen1.bin"
screen2
!bin "screen2.bin"
screen3
!bin "screen3.bin"
screen4
!bin "screen4.bin"
screen5
!bin "screen5.bin"
